/**************************************************************************************************
 *     CVS $Id: keycode.h,v 1.13 2003/12/12 22:02:02 dvhart Exp $
 * DESCRIPTION: An enumeration of all keys and modifiers, used by key_event.
 *     AUTHORS: Darren Hart
 *  START DATE: 2003/Feb/22
 *
 *   COPYRIGHT: 2003 by Darren Hart, Vernon Mauery, Marc Strmke, Dirk Hrner
 *     LICENSE: This software is licenced under the Libstk license available with the source as 
 *              license.txt or at http://www.libstk.org/index.php?page=docs/license
 *************************************************************************************************/

#ifndef STK_KEYCODE_H
#define STK_KEYCODE_H

namespace stk
{
    enum modcode
    {
        mod_none                = 0x0000,
        mod_numlock             = 0x0001,
        mod_capslock            = 0x0002,
        mod_scrolllock          = 0x0004,
        mod_rightshift          = 0x0008,
        mod_leftshift           = 0x0010,
        mod_rightcontrol        = 0x0020,
        mod_leftcontrol         = 0x0040,
        mod_rightalt            = 0x0080,
        mod_leftalt             = 0x0100,
        mod_rightmeta           = 0x0200,
        mod_leftmeta            = 0x0400,
        mod_mode                = 0x0800,  // "Alt Gr" key
        // left/right logical or's
        mod_shift               = mod_rightshift | mod_leftshift,
        mod_control             = mod_rightcontrol | mod_leftcontrol,
        mod_alt                 = mod_rightalt | mod_leftalt,
        mod_meta                = mod_rightmeta | mod_leftmeta
    };
    
    enum keycode
    {
        // ASCII mapped keysyms
        key_null                = 0,  
        key_backspace           = 8,
        key_tab                 = 9,
        key_clear               = 12,  // what is this ?
        key_enter               = 13,
        key_pause               = 19,
        key_escape              = 27,
        key_space               = 32,
        key_exclamation         = 33,
        key_doublequote         = 34,
        key_pound               = 35,
        key_dollar              = 36,
        key_percent             = 37,
        key_ampersand           = 38,
        key_quote               = 39,
        key_leftparen           = 40,
        key_rightparen          = 41,
        key_asterik             = 42,
        key_plus                = 43,
        key_comma               = 44,
        key_minus               = 45,
        key_period              = 46,
        key_slash               = 47,
        key_0                   = 48,
        key_1                   = 49,
        key_2                   = 50,
        key_3                   = 51,
        key_4                   = 52,
        key_5                   = 53,
        key_6                   = 54,
        key_7                   = 55,
        key_8                   = 56,
        key_9                   = 57,
        key_colon               = 58,
        key_semicolon           = 59,
        key_less                = 60,
        key_equal               = 61,
        key_greater             = 62,
        key_question            = 63,
        key_at                  = 64,
        
        // skip uppercase letters
        
        key_leftbracker         = 91,
        key_backslash           = 92,
        key_rightbracket        = 93,
        key_caret               = 94,
        key_underscore          = 95,
        key_tick                = 96,
        key_a                   = 97,
        key_b                   = 98,
        key_c                   = 99,
        key_d                   = 100,
        key_e                   = 101,
        key_f                   = 102,
        key_g                   = 103,
        key_h                   = 104,
        key_i                   = 105,
        key_j                   = 106,
        key_k                   = 107,
        key_l                   = 108,
        key_m                   = 109,
        key_n                   = 110,
        key_o                   = 111,
        key_p                   = 112,
        key_q                   = 113,
        key_r                   = 114,
        key_s                   = 115,
        key_t                   = 116,
        key_u                   = 117,
        key_v                   = 118,
        key_w                   = 119,
        key_x                   = 120,
        key_y                   = 121,
        key_z                   = 122,
        key_leftbrace           = 123,
        key_pipe                = 124,
        key_rightbrace          = 125,
        key_tilde               = 126,
        key_delete              = 127,
        // end of ASCII mapped keysyms

        // international keysyms (ala SDL and X11)
        key_world_0             = 160,  // 0xA0
        key_world_1             = 161,
        key_world_2             = 162,
        key_world_3             = 163,
        key_world_4             = 164,
        key_world_5             = 165,
        key_world_6             = 166,
        key_world_7             = 167,
        key_world_8             = 168,
        key_world_9             = 169,
        key_world_10            = 170,
        key_world_11            = 171,
        key_world_12            = 172,
        key_world_13            = 173,
        key_world_14            = 174,
        key_world_15            = 175,
        key_world_16            = 176,
        key_world_17            = 177,
        key_world_18            = 178,
        key_world_19            = 179,
        key_world_20            = 180,
        key_world_21            = 181,
        key_world_22            = 182,
        key_world_23            = 183,
        key_world_24            = 184,
        key_world_25            = 185,
        key_world_26            = 186,
        key_world_27            = 187,
        key_world_28            = 188,
        key_world_29            = 189,
        key_world_30            = 190,
        key_world_31            = 191,
        key_world_32            = 192,
        key_world_33            = 193,
        key_world_34            = 194,
        key_world_35            = 195,
        key_world_36            = 196,
        key_world_37            = 197,
        key_world_38            = 198,
        key_world_39            = 199,
        key_world_40            = 200,
        key_world_41            = 201,
        key_world_42            = 202,
        key_world_43            = 203,
        key_world_44            = 204,
        key_world_45            = 205,
        key_world_46            = 206,
        key_world_47            = 207,
        key_world_48            = 208,
        key_world_49            = 209,
        key_world_50            = 210,
        key_world_51            = 211,
        key_world_52            = 212,
        key_world_53            = 213,
        key_world_54            = 214,
        key_world_55            = 215,
        key_world_56            = 216,
        key_world_57            = 217,
        key_world_58            = 218,
        key_world_59            = 219,
        key_world_60            = 220,
        key_world_61            = 221,
        key_world_62            = 222,
        key_world_63            = 223,
        key_world_64            = 224,
        key_world_65            = 225,
        key_world_66            = 226,
        key_world_67            = 227,
        key_world_68            = 228,
        key_world_69            = 229,
        key_world_70            = 230,
        key_world_71            = 231,
        key_world_72            = 232,
        key_world_73            = 233,
        key_world_74            = 234,
        key_world_75            = 235,
        key_world_76            = 236,
        key_world_77            = 237,
        key_world_78            = 238,
        key_world_79            = 239,
        key_world_80            = 240,
        key_world_81            = 241,
        key_world_82            = 242,
        key_world_83            = 243,
        key_world_84            = 244,
        key_world_85            = 245,
        key_world_86            = 246,
        key_world_87            = 247,
        key_world_88            = 248,
        key_world_89            = 249,
        key_world_90            = 250,
        key_world_91            = 251,
        key_world_92            = 252,
        key_world_93            = 253,
        key_world_94            = 254,
        key_world_95            = 255,  // 0xFF

        // numeric keypad 
        key_kp0                 = 256,
        key_kp1                 = 257,
        key_kp2                 = 258,
        key_kp3                 = 259,
        key_kp4                 = 260,
        key_kp5                 = 261,
        key_kp6                 = 262,
        key_kp7                 = 263,
        key_kp8                 = 264,
        key_kp9                 = 265,
        key_kp_period           = 266,
        key_kp_divide           = 267,
        key_kp_multiply         = 268,
        key_kp_minus            = 269,
        key_kp_plus             = 270,
        key_kp_enter            = 271,
        key_kp_equal            = 272,
        
        // arrows + home/end pad
        key_uparrow             = 273,
        key_downarrow           = 274,
        key_rightarrow          = 275,
        key_leftarrow           = 276,
        key_insert              = 277,
        key_home                = 278,
        key_end                 = 279,
        key_pageup              = 280,
        key_pagedown            = 281,
        
        // function keys
        key_f1                  = 282,
        key_f2                  = 283,
        key_f3                  = 284,
        key_f4                  = 285,
        key_f5                  = 286,
        key_f6                  = 287,
        key_f7                  = 288,
        key_f8                  = 289,
        key_f9                  = 290,
        key_f10                 = 291,
        key_f11                 = 292,
        key_f12                 = 293,
        key_f13                 = 294,
        key_f14                 = 295,
        key_f15                 = 296,
        
        // modifier keys
        key_numlock             = 300,
        key_capslock            = 301,
        key_scrolllock          = 302,
        key_rightshift          = 303,
        key_leftshift           = 304,
        key_rightcontrol        = 305,
        key_leftcontrol         = 306,
        key_rightalt            = 307,
        key_leftalt             = 308,
        key_rightmeta           = 309,
        key_leftmeta            = 310,
        key_leftsuper           = 311,  // Left "Windows" key
        key_rightsuper          = 312,  // Right "Windows" key
        key_mode                = 313,  // "Alt Gr" key
        key_compose             = 314,  // Multi-key compose key
        
        // miscellaneous
        key_help                = 315,
        key_print               = 316,
        key_sysreq              = 317,
        key_break               = 318,
        key_menu                = 319,
        key_power               = 320,  // Power Macintosh power key
        key_euro                = 321,  // Some european keyboards
        key_undo                = 322,  // Atari keyboard has Undo
        key_email               = 323,
        key_www                 = 324,
        key_sleep               = 325,
        key_media               = 326,
        key_prev                = 327,
        key_next                = 328,
        key_pauseplay           = 329,
        key_play                = 330,
        key_fastforward         = 331,
        key_rewind              = 332,
        key_mute                = 333,
        key_volumeup            = 334,
        key_volumedown          = 335,
        key_search              = 336,
        key_homedir             = 337,
        
        // infra red remote control keys 
        // numeric
        key_ir_0                = 338,
        key_ir_1                = 339,
        key_ir_2                = 340,
        key_ir_3                = 341,
        key_ir_4                = 342,
        key_ir_5                = 343,
        key_ir_6                = 344,
        key_ir_7                = 345,
        key_ir_8                = 346,
        key_ir_9                = 347,
        key_ir_plus10           = 348,
        // source select
        key_ir_input            = 349,
        key_ir_aux1             = 350,
        key_ir_aux2             = 351,
        key_ir_cable            = 352,
        key_ir_cd1              = 353,
        key_ir_cd2              = 354,
        key_ir_dvd1             = 355,
        key_ir_dvd2             = 356,
        key_ir_md1              = 357,
        key_ir_md2              = 358,
        key_ir_tv               = 359,
        key_ir_sat              = 360,
        key_ir_tuner            = 361,
        key_ir_vcr1             = 362,
        key_ir_vcr2             = 363,
        // media control
        key_ir_play             = 364,
        key_ir_pause            = 365,
        key_ir_stop             = 366,
        key_ir_next             = 367,
        key_ir_prev             = 368,
        key_ir_fastforward      = 369,
        key_ir_rewind           = 370,
        key_ir_slowforward      = 371,
        key_ir_slowrewind       = 372,
        key_ir_record           = 373,
        // menu control
        key_ir_menu             = 374,
        key_ir_up               = 375,
        key_ir_down             = 376,
        key_ir_right            = 377,
        key_ir_left             = 378,
        key_ir_ok               = 379,
        key_ir_enter            = 380,
        key_ir_select           = 381,
        key_ir_back             = 382,
        // channel and volume
        key_ir_channelup        = 383,
        key_ir_channeldown      = 384,
        key_ir_prevchannel      = 385,
        key_ir_volumeup         = 386,
        key_ir_volumedown       = 387,
        key_ir_mute             = 388,
        key_ir_tvvolumeup       = 389,
        key_ir_tvvolumedown     = 390,
        // power related
        key_ir_power            = 391,
        key_ir_avpower          = 392,
        key_ir_tvpower          = 393,
        key_ir_standby          = 394,
        // miscellaneous
        key_ir_angle            = 395,
        key_ir_antenna          = 396,
        key_ir_audio            = 397,
        key_ir_av               = 398,
        key_ir_clear            = 399,
        key_ir_display          = 400,
        key_ir_dsp              = 401,
        key_ir_effect           = 402,
        key_ir_eject            = 403,
        key_ir_guide            = 404,
        key_ir_info             = 405,
        key_ir_pip              = 406,
        key_ir_program          = 407,
        key_ir_random           = 408,
        key_ir_repeat           = 409,
        key_ir_repeatab         = 410,
        key_ir_reset            = 411,
        key_ir_search           = 412,
        key_ir_setup            = 413,
        key_ir_subtitle         = 414,
        key_ir_swap             = 415,
        key_ir_topmenu          = 416,
        key_ir_zoom             = 417,
        // end infra red keys
        
        // define other keys here
        // ...

        key_unknown
    };
} // namespace stk

#endif

